﻿package widget.transitions
{
	import flash.display.*;	
	import flash.events.*
	import flash.geom.*;
	
	import org.papervision3d.materials.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.cameras.Camera3D;	
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;	
	
	import gs.TweenMax;
	import gs.easing.*;		
	
	public class Cubes3D extends MovieClip implements ITransition
	{
		private var viewport:Viewport3D; // The Viewport
		private var renderer:BasicRenderEngine; // Rendering engine
		private var scene:Scene3D; // A Scene	
		private var camera:Camera3D;		
		
		private var cubeSize:Number = 100;
		private var horizontalAmount:Number = 4;
		private var verticalAmount:Number = 3;
		
		public function Cubes3D()
		{
		}
		
		public function doTransition(oldPhotoBitmapData:BitmapData, newPhotoBitmapData:BitmapData):void
		{
			init3d();
			
			for(var v:int = 0; v < verticalAmount; v++)
			{
				for(var h:int = 0; h < horizontalAmount; h++)
				{
					var oldPhotoPart:BitmapData = new BitmapData(cubeSize, cubeSize);
					oldPhotoPart.copyPixels(oldPhotoBitmapData ,new Rectangle(h*cubeSize, v*cubeSize, cubeSize, cubeSize), new Point(0,0));
					
					var newPhotoPart:BitmapData = new BitmapData(cubeSize, cubeSize);
					newPhotoPart.copyPixels(newPhotoBitmapData ,new Rectangle(h*cubeSize, v*cubeSize, cubeSize, cubeSize), new Point(0,0));					
					
					var frontMaterial:BitmapMaterial = new BitmapMaterial(oldPhotoPart);
					var backMaterial:BitmapMaterial = new BitmapMaterial(newPhotoPart);
					
					var ml:MaterialsList = new MaterialsList({
															 	front:	frontMaterial, 
																right:	new ColorMaterial(0x000000, 0), 
																left:	new ColorMaterial(0x000000, 0), 
																back:   backMaterial,
																top:    new ColorMaterial(0x000000, 0),
																bottom: new ColorMaterial(0x000000, 0)
															});					
					
					var cube:Cube = new Cube(ml, cubeSize, cubeSize, cubeSize, 4, 4, 4);
					
					cube.name = "cube_"+v+"_"+h;
					
					cube.x = -(h * cubeSize) + (stage.stageWidth / 2) - (cubeSize / 2);
					cube.y = -(v * cubeSize) + (stage.stageHeight / 2) - (cubeSize / 2);
									
					scene.addChild(cube);
				}
			}
			
			// cubes are loaded, so start rotating!
			for(var i:int = 0; i < verticalAmount; i++)
			{
				for(var j:int = 0; j < horizontalAmount; j++)
				{
					var cubeNumber:Number = i*verticalAmount + j;
					
					viewport.getChildLayer(scene.getChildByName("cube_"+i+"_"+j), true).layerIndex = cubeNumber;
					
					// but not all at once, so use a delayedcall
					TweenMax.delayedCall(cubeNumber/10, rotateCube, ["cube_"+i+"_"+j]);
				}
			}
		}
		
		private function rotateCube(cubeName:String)
		{
			TweenMax.to(scene.getChildByName(cubeName), 0.75, {rotationY: -180, ease:Linear.easeNone, onComplete: sortLayers, onCompleteParams: [cubeName]});
		}
		
		private function sortLayers(cubeName:String)
		{
			viewport.getChildLayer(scene.getChildByName(cubeName), true).layerIndex = 1;
		}
		
		private function init3d()
		{
			viewport = new Viewport3D(stage.stageHeight, stage.stageWidth, true);
			renderer = new BasicRenderEngine();
			scene = new Scene3D();
			camera = new Camera3D();
		
			var cameraPoint:DisplayObject3D = new DisplayObject3D();
			
			cameraPoint.x = 0;
			cameraPoint.y = 0;
			cameraPoint.z = 0;

			camera.z = 2450;
			camera.zoom = 120;
			camera.focus = 20;
			
			camera.target = cameraPoint;

			addEventListener(Event.ENTER_FRAME, render);
			
			addChild(viewport);			
		}
		
		private function render(e:Event):void
		{
			renderer.renderScene(scene, camera, viewport);
		}
		
		public function destroyTransition():void
		{
			removeEventListener(Event.ENTER_FRAME, render);

			removeChild(viewport);
			
			viewport = null;
			renderer = null;
			scene = null;
			camera = null;
			cameraPoint = null;
		}
	}
}